use crate::server::ServerProcess;
use flate2::write::GzEncoder;
use flate2::Compression;
use std::fs::File;
use std::io;
use std::io::Write;
use std::path::{Path, PathBuf};
use std::process::{Child, Command, Stdio};

pub struct ServerCommand {
    java: String,
    jar: PathBuf,
    config_dir: PathBuf,
    world_dir: PathBuf,
    xms: u32,
    xmx: u32,
    child: Option<Child>,
}

impl ServerCommand {
    pub fn new(jar: &str, config_dir: &str, world_dir: &str) -> Self {
        ServerCommand {
            java: String::from("java"),
            jar: Path::new(jar).canonicalize().unwrap(),
            config_dir: Path::new(config_dir).canonicalize().unwrap(),
            world_dir: Path::new(world_dir).canonicalize().unwrap(),
            xms: 1024,
            xmx: 2048,
            child: None,
        }
    }

    pub fn java(mut self, java: &str) -> Self {
        self.java = String::from(java);

        self
    }

    pub fn mem(mut self, xms: u32, xmx: u32) -> Self {
        self.xms = xms;
        self.xmx = xmx;

        self
    }

    fn accept_eula(&self) -> std::io::Result<()> {
        let mut eula_path = self.config_dir.clone();
        eula_path.push("eula.txt");
        let mut eula_file = File::create(eula_path)?;
        eula_file.write(b"eula=true")?;

        Ok(())
    }

    pub fn spawn(&mut self) -> std::io::Result<ServerProcess> {
        self.accept_eula()?;

        let child = Command::new(&self.java)
            .current_dir(&self.config_dir)
            .arg("-jar")
            .arg(&self.jar)
            .arg("--universe")
            .arg(&self.world_dir)
            .arg("--nogui")
            .stdin(Stdio::piped())
            .spawn()?;

        Ok(ServerProcess::new(child))
    }
}
