use flate2::write::GzEncoder;
use flate2::Compression;
use std::fs::File;
use std::io;
use std::io::Write;
use std::path::{Path, PathBuf};
use std::process::Child;
use std::process::{Command, Stdio};

pub struct ServerProcess {
    child: Child,
}

impl ServerProcess {
    pub fn new(child: Child) -> ServerProcess {
        ServerProcess { child }
    }

    pub fn send_command(&mut self, cmd: &str) {
        match cmd.trim() {
            "stop" | "exit" => self.stop(),
            "backup" => self.backup(),
            s => self.custom(s),
        }
    }

    fn custom(&mut self, cmd: &str) {
        let mut stdin = self.child.stdin.take().unwrap();
        stdin
            .write_all(format!("{}\n", cmd.trim()).as_bytes())
            .unwrap();
        stdin.flush().unwrap();
    }

    pub fn stop(&mut self) {
        self.custom("stop");
        self.child.wait();
    }

    pub fn backup(&mut self) {
        // Make sure the server isn't modifying the files during the backup
        self.custom("save-off");
        self.custom("save-all");

        // Create a gzip-compressed tarball of the worlds folder
        let filename = format!(
            "{}",
            chrono::offset::Local::now().format("backups/%Y-%m-%d_%H-%M-%S.tar.gz")
        );
        let tar_gz = std::fs::File::create(filename).unwrap();
        let enc = GzEncoder::new(tar_gz, Compression::default());
        let mut tar = tar::Builder::new(enc);
        tar.append_dir_all("worlds", "worlds").unwrap();

        self.custom("save-on");
    }
}
